/* Copyright (c) 2012 Nathan Osman

   Permission is hereby granted, free of charge, to any person obtaining a copy
   of this software and associated documentation files (the "Software"), to
   deal in the Software without restriction, including without limitation the
   rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
   sell copies of the Software, and to permit persons to whom the Software is
   furnished to do so, subject to the following conditions:

   The above copyright notice and this permission notice shall be included in
   all copies or substantial portions of the Software.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
   AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
   LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
   FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
   IN THE SOFTWARE. */

#ifndef CAPI_H
#define CAPI_H

// Qt includes
#include <QNetworkAccessManager>
#include <QNetworkReply>

// StackApplet includes
#include <CInboxItem.h>

// Makes network requests on behalf of the user
// who has authorized the application to do so.
class CAPI : public QNetworkAccessManager
{
    Q_OBJECT

public:

    CAPI();

    void SetAccessToken(QString access_token) { m_access_token = access_token; }

    static const QString APIKey;
    static const quint32 ClientID;

public slots:

    void OnReply(QNetworkReply *);

    // Polls the /inbox route
    void Update();

signals:

    // Emitted when an error occurs
    void Error(QString);

    // Emitted when inbox items are returned in the response
    void InboxItems(CInboxItemList);

private:

    // Information necessary to connect to the API.
    QString m_access_token;

};

#endif // CAPI_H
