/* Copyright (c) 2012 Nathan Osman

   Permission is hereby granted, free of charge, to any person obtaining a copy
   of this software and associated documentation files (the "Software"), to
   deal in the Software without restriction, including without limitation the
   rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
   sell copies of the Software, and to permit persons to whom the Software is
   furnished to do so, subject to the following conditions:

   The above copyright notice and this permission notice shall be included in
   all copies or substantial portions of the Software.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
   AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
   LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
   FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
   IN THE SOFTWARE. */

#ifndef CSTACKAPPLET_H
#define CSTACKAPPLET_H

// Qt includes
#include <QObject>
#include <QTimer>

// StackApplet includes
#include <CAPI.h>
#include <CInboxItem.h>
#include <CTrayIcon.h>
#include <dialogs/CAuthDialog.h>
#include <dialogs/CInfoPopup.h>

class CStackApplet : public QObject
{
    Q_OBJECT

public:

    CStackApplet();
    ~CStackApplet();

private slots:

    void OnAbout();
    void OnAccessToken(QString);
    void OnIconClicked(QSystemTrayIcon::ActivationReason);
    void OnInboxItems(CInboxItemList);
    void OnMarkAllRead();
    void OnPreferences();

private:

    CAPI        m_api;
    CAuthDialog m_auth;
    CInfoPopup  m_popup;
    CTrayIcon   m_icon;

    // The timer controlling the updates
    QTimer m_timer;

    // Whether or not we have a valid access token
    bool m_have_access;

    // The current list of unread items
    CInboxItemList m_unread;

    // The 'link' field of the last inbox item seen by StackApplet
    // and the last one that the user has marked as read
    QString m_last_link_seen,
            m_last_link_read;

};

#endif // CSTACKAPPLET_H
