/* Copyright (c) 2012 Nathan Osman

   Permission is hereby granted, free of charge, to any person obtaining a copy
   of this software and associated documentation files (the "Software"), to
   deal in the Software without restriction, including without limitation the
   rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
   sell copies of the Software, and to permit persons to whom the Software is
   furnished to do so, subject to the following conditions:

   The above copyright notice and this permission notice shall be included in
   all copies or substantial portions of the Software.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
   AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
   LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
   FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
   IN THE SOFTWARE. */

#ifndef CTRAYICON_H
#define CTRAYICON_H

// Qt includes
#include <QAction>
#include <QMenu>
#include <QSystemTrayIcon>

class CTrayIcon : public QSystemTrayIcon
{
    Q_OBJECT

public:

    CTrayIcon();
    ~CTrayIcon();

public slots:

    // Displays a notification for the specified events
    void DisplayError(QString);
    void DisplayNotification(QString, QString);

    // Activate / deactivate the icon indicating that there are new messages
    void ShowMessageIcon();
    void ShowNoMessageIcon();

signals:

    void About();
    void Authorize();
    void Deauthorize();
    void Preferences();
    void Quit();

private:

    // Sets up the context menu
    void SetupMenu();

    // The menu to be displayed when the user right-clicks the icon
    QMenu * m_context_menu;

    // Items in the menu
    QAction * m_authorize,
            * m_deauthorize;

};

#endif // CTRAYICON_H
