/* Copyright (c) 2012 Nathan Osman

   Permission is hereby granted, free of charge, to any person obtaining a copy
   of this software and associated documentation files (the "Software"), to
   deal in the Software without restriction, including without limitation the
   rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
   sell copies of the Software, and to permit persons to whom the Software is
   furnished to do so, subject to the following conditions:

   The above copyright notice and this permission notice shall be included in
   all copies or substantial portions of the Software.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
   AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
   LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
   FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
   IN THE SOFTWARE. */

#include <CAPI.h>

// Qt includes
#include <qjson/parser.h>
#include <qjson/qobjecthelper.h>
#include <QNetworkRequest>
#include <QSharedPointer>
#include <QUrl>
#include <QVariant>

// Store the API key and client ID
QString const CAPI::APIKey   = "VN8jyf7k8STlkO3lx*iw6w((";
quint32 const CAPI::ClientID = 120;

CAPI::CAPI()
{
    connect(this, SIGNAL(finished(QNetworkReply*)), SLOT(OnReply(QNetworkReply*)));
}

void CAPI::OnReply(QNetworkReply * reply)
{
    // Begin by parsing the JSON returned
    QJson::Parser parser; bool ok = false;
    QVariant json = parser.parse(reply->readAll(), &ok);

    // Ensure the JSON parser succeeded
    if(ok)
    {
        // Make sure that error_message wasn't supplied
        QVariantMap map = json.toMap();
        if(!map.contains("error_message"))
        {
            // Now ensure that 'items' was supplied
            if(map.contains("items"))
            {
                // Enumerate each item, converting it to a CInboxItem and
                // adding it to the list
                CInboxItemList inbox_list;
                foreach(QVariant json_item, map["items"].toList())
                {
                    // Create the object
                    QSharedPointer<CInboxItem> inbox_item(new CInboxItem());
                    QJson::QObjectHelper::qvariant2qobject(json_item.toMap(), inbox_item.data());

                    // Add it to the list
                    inbox_list.append(inbox_item);
                }

                // Emit a signal with the list
                emit InboxItems(inbox_list);
            }
            else
                emit Error(tr("'items' missing from JSON object returned by the API server"));
        }
        else
            emit Error(map["error_message"].toString());
    }
    else
        emit Error(tr("JSON data received from the API could not be decoded"));

    reply->deleteLater();
}

void CAPI::Update()
{
    // Construct the URL for the request
    QUrl url("https://api.stackexchange.com/2.0/inbox");
    url.addQueryItem("key", APIKey);
    url.addQueryItem("access_token", m_access_token);

    // Create the request and issue it
    get(QNetworkRequest(url));
}
