/* Copyright (c) 2012 Nathan Osman

   Permission is hereby granted, free of charge, to any person obtaining a copy
   of this software and associated documentation files (the "Software"), to
   deal in the Software without restriction, including without limitation the
   rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
   sell copies of the Software, and to permit persons to whom the Software is
   furnished to do so, subject to the following conditions:

   The above copyright notice and this permission notice shall be included in
   all copies or substantial portions of the Software.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
   AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
   LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
   FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
   IN THE SOFTWARE. */

#include <CTrayIcon.h>

// Qt includes
#include <QIcon>

CTrayIcon::CTrayIcon()
    : m_context_menu(NULL), m_authorize(NULL), m_deauthorize(NULL)
{
    // Set up the menu
    SetupMenu();

    // Set the icon, tooltip, and menu
    ShowNoMessageIcon();
    setToolTip(tr("StackApplet"));
    setContextMenu(m_context_menu);

    // Display the icon
    show();
}

CTrayIcon::~CTrayIcon()
{
    if(m_context_menu)
        delete m_context_menu;
}

void CTrayIcon::DisplayError(QString error_message)
{
    // Display the error message and display the error icon
    showMessage(tr("An Error Has Occurred"), tr("Details: %1").arg(error_message), Critical);
    setIcon(QIcon(":/icons/error.png"));
}

void CTrayIcon::DisplayNotification(QString title, QString body)
{
    showMessage(title, body, Information);
}

void CTrayIcon::ShowMessageIcon()
{
    setIcon(QIcon(":/icons/messages.png"));
}

void CTrayIcon::ShowNoMessageIcon()
{
    setIcon(QIcon(":/icons/no_messages.png"));
}

void CTrayIcon::SetupMenu()
{
    // Generate the menu
    m_context_menu = new QMenu();

    QMenu * auth_menu = new QMenu(tr("Authorization"), m_context_menu);

    m_authorize   = new QAction(tr("Authorize"),   auth_menu);
    m_deauthorize = new QAction(tr("Deauthorize"), auth_menu);

    connect(m_authorize,   SIGNAL(triggered()), SIGNAL(Authorize()));
    connect(m_deauthorize, SIGNAL(triggered()), SIGNAL(Deauthorize()));

    auth_menu->addAction(m_authorize);
    auth_menu->addAction(m_deauthorize);

    m_context_menu->addMenu(auth_menu);

    m_context_menu->addSeparator();
    m_context_menu->addAction(tr("Preferences"),       this, SIGNAL(Preferences()));
    m_context_menu->addAction(tr("About StackApplet"), this, SIGNAL(About()));
    m_context_menu->addSeparator();

    m_context_menu->addAction(tr("Quit"), this, SIGNAL(Quit()));
}
