/* Copyright (c) 2012 Nathan Osman

   Permission is hereby granted, free of charge, to any person obtaining a copy
   of this software and associated documentation files (the "Software"), to
   deal in the Software without restriction, including without limitation the
   rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
   sell copies of the Software, and to permit persons to whom the Software is
   furnished to do so, subject to the following conditions:

   The above copyright notice and this permission notice shall be included in
   all copies or substantial portions of the Software.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
   AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
   LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
   FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
   IN THE SOFTWARE. */

#include <dialogs/CAuthDialog.h>
#include "ui_CAuthDialog.h"

// StackApplet includes
#include <CAPI.h>

CAuthDialog::CAuthDialog()
    : ui(new Ui::CAuthDialog)
{
    ui->setupUi(this);

    // Enable local storage
    ui->WebView->settings()->setAttribute(QWebSettings::LocalStorageEnabled, true);

    // Set up the signals and slots
    connect(ui->WebView, SIGNAL(loadProgress(int)), ui->Progress, SLOT(setValue(int)));
    connect(ui->WebView, SIGNAL(urlChanged(QUrl)), SLOT(OnPageChanged(QUrl)));
}

CAuthDialog::~CAuthDialog()
{
    delete ui;
}

void CAuthDialog::Authorize()
{
    // Build the URL for authorization
    QUrl auth_url("https://stackexchange.com/oauth/dialog");
    auth_url.addQueryItem("client_id",    tr("%1").arg(CAPI::ClientID));
    auth_url.addQueryItem("scope",        "read_inbox,no_expiry");
    auth_url.addQueryItem("redirect_uri", "https://stackexchange.com/oauth/login_success");

    // Set the WebView's page to the auth page
    ui->WebView->setUrl(auth_url);

    exec();
}

void CAuthDialog::OnPageChanged(const QUrl & url)
{
    // Set the contents of the line edit
    ui->CurrentURL->setText(url.toString());
    ui->CurrentURL->setCursorPosition(0);

    // Only perform further action if we have reached the OAuth success page
    if(url.toString(QUrl::RemoveFragment) == "https://stackexchange.com/oauth/login_success")
    {
        // This is a neat trick - set the query string to the value
        // of the fragment (hash) so that we can parse it with the
        // query string-parsing functions.
        QUrl new_url;
        new_url.setEncodedQuery(url.encodedFragment());

        // Check to see if an error was encountered
        QString error_description = new_url.queryItemValue("error_description");
        if(!error_description.isEmpty())
        {
            // Replace the + signs with spaces and emit the string as an error
            emit Error(error_description.replace('+', ' '));

            // Reject the dialog and return
            reject(); return;
        }

        // Check to make sure the access token was specified
        QString access_token = new_url.queryItemValue("access_token");
        if(!access_token.isEmpty())
        {
            // Emit the signal with the access token
            emit AccessToken(access_token);

            // Accept the dialog and return
            accept(); return;
        }

        // If neither an error OR an access token was supplied, then that is itself an error
        emit Error(tr("neither an error message nor an access token was specified in the response"));
        reject();
    }
}
